﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Timers;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace tetris
{
    /// <summary>
    /// Background.xaml 的交互逻辑
    /// </summary>
    public partial class TetrisBoard : UserControl
    {
        Timer timer = new Timer();
        int level = 1;
        int removed = 0;
        int score = 0;
        int levelScore = 0;

        public TetrisShape Shape { get; set; }

        public TetrisShape Next { get; set; }

        private bool[,] datas = new bool[20, 10];
        private Cube[,] cubes = new Cube[20, 10];

        public TetrisBoard()
        {
            InitializeComponent();

            this.Shape = new Square();
            Shape.GridData = datas;
            Shape.Left = 4;
            this.Next = new Square();
            Shape.ReachedBottom += () =>
            {
                Remove();
                this.Dispatcher.Invoke(() =>
                {
                    rowsText.Text = removed.ToString();
                    scoreText.Text = score.ToString();
                    levelText.Text = level.ToString();
                });
                if (Shape.IsFull())
                {
                    timer.Stop();
                    MessageBox.Show("Game over!");
                }

                Paint();
            };
            timer.Interval = 1000;

            timer.Elapsed += (s, e) =>
            {
                Shape.MoveDown();
                Paint();
            };
        }
        public void Remove()
        {
            int removeCount = 0;
            for (var row = 0; row < 20; row++)
            {
                bool full = true;
                for (var col = 0; col < 10; col++)
                {
                    full = full && datas[row, col];
                }
                if (full)
                {
                    removed++;
                    removeCount++;
                    for (var ui = row - 1; ui > 0; ui--)
                    {
                        for (var uj = 0; uj < 10; uj++)
                        {
                            datas[ui + 1, uj] = datas[ui, uj]; //后一行等于前一行。
                        }
                    }
                }
            }

            var tempScore = 0;
            if (removeCount == 1) tempScore += 100;
            if (removeCount == 2) tempScore += 200;
            if (removeCount == 3) tempScore += 400;
            if (removeCount == 4) tempScore += 400;

            score += tempScore;
            levelScore += tempScore;

            if (levelScore >= 4000)
            {
                if (level < 12)
                    level++;
                else
                    level = 1;

                levelScore = 0;
            }
        }
        protected void Paint()
        {
            var arr = Shape.ShapeArray;
            for (int row = 0; row < 20; row++)
            {
                for (int col = 0; col < 10; col++)
                {
                    bool show = datas[row, col];
                    Cube cube = cubes[row, col];
                    cube.Dispatcher.Invoke(() =>
                    {
                        if (show)
                        {
                            cube.Visibility = Visibility.Visible;
                        }
                        else
                        {
                            cube.Visibility = Visibility.Hidden;
                        }
                    });
                }
            }

            for (int row = 0; row < arr.GetLength(0); row++)
            {
                for (var col = 0; col < arr.GetLength(1); col++)
                {
                    if (arr[row, col])
                        this.Dispatcher.Invoke(() =>
                        {
                            var cube = cubes[Shape.Top + row, Shape.Left + col];
                            cube.Background = new SolidColorBrush(Shape.ShapeColor);
                            cube.Visibility = Visibility.Visible;
                        });

                }
            }
        }

        public void Upgrade()
        {
            if (level < 12)
                level++;
            else
                level = 1;

            int ticks = 1000 / level;
            timer.Interval = ticks;
        }

        public void Start()
        {
            timer.Stop();
            for (var row = 0; row < 20; row++)
            {
                for (var col = 0; col < 10; col++)
                {
                    this.datas[row, col] = false;
                }
            }
            Shape.RandomShape();
            Paint();
            timer.Start();
        }

        public void Stop()
        {
            timer.Stop();
        }

        public void Pause()
        {
            timer.Enabled = !timer.Enabled;
        }

        private void UserControl_Initialized(object sender, EventArgs e)
        {
            for (var col = 0; col <= 10; col++)
            {
                Line line = new Line();
                line.X1 = col * 30;
                line.X2 = col * 30;
                line.Y1 = 0;
                line.Y2 = 600;
                line.Stroke = new SolidColorBrush(Color.FromArgb(0xFF, 0, 0, 1));
                grid.Children.Add(line);
            }

            for (var row = 0; row <= 20; row++)
            {
                Line line = new Line();
                line.X1 = 0;
                line.X2 = 300;
                line.Y1 = row * 30;
                line.Y2 = row * 30;
                line.Stroke = new SolidColorBrush(Color.FromArgb(0xFF, 0, 0, 1));
                grid.Children.Add(line);
            }

            for (int row = 0; row < 20; row++)
            {
                for (int col = 0; col < 10; col++)
                {
                    bool show = datas[row, col];
                    Cube cube = new Cube();
                    cube.Width = 24;
                    cube.Height = 24;
                    cube.HorizontalAlignment = HorizontalAlignment.Left;
                    cube.VerticalAlignment = VerticalAlignment.Top;
                    if (show)
                    {
                        cube.Visibility = Visibility.Visible;
                    }
                    else
                    {
                        cube.Visibility = Visibility.Hidden;
                    }
                    var m = cube.Margin;
                    cube.Margin = new Thickness(m.Left + col * 30 + 3, m.Top + row * 30 + 3, m.Right, m.Bottom);
                    cubes[row, col] = cube;
                    grid.Children.Add(cube);
                }
            }
        }

        public void ProcessKey(Key key)
        {
            if (key == Key.Left) { Shape.MoveLeft(); }
            if (key == Key.Right) { Shape.MoveRight(); }
            if (key == Key.Space) { Shape.Drop(); }
            if (key == Key.Up) { Shape.ChangeDirection(); }
            if (key == Key.Down) { Shape.MoveQuickly(); }
            Paint();
        }
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            scoreText.Text = "0";
            rowsText.Text = "0";
            levelText.Text = "0";
            this.Start();
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            Pause();
        }
    }
}
