﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;

namespace tetris
{
    public class TetrisShape
    {
        public TetrisShape()
        {
            Random random = new Random();
            RandomShape();
        }

        bool[][,] baseShapes = new bool[][,]{
            new bool[,]{{ true, true },  { true, true} },
            new bool[,]{{true},{true}, {true}, {true}},
            new bool[,]{{false, true, true}, {true, true, false}},
            new bool[,]{{true, true, false}, {false, true, true}},
            new bool[,]{{true, true, true}, {true, false, false}},
            new bool[,]{{true, true, true}, {false, false, true}},
           new bool[,] { { false, true, false }, {true, true, true}}
        };

        Color[] colors ={
            Color.FromRgb(0xFF,0,0),
            Color.FromRgb(0,0xFF,0),
            Color.FromRgb(0,0,0xFF),
            Color.FromRgb(0xFF,0xFF,0),
            Color.FromRgb(0xFF,0,0xFF),
            Color.FromRgb(0,0xFF,0xFF),
            Color.FromRgb(0xEE,0xEE,0xEE)
    };

        public Color ShapeColor { get; set; }

        public bool[,] GridData { get; set; }

        public int Left { get; set; }

        public int Top { get; set; }

        public bool[,] ShapeArray { get; set; }

        public void RandomShape()
        {
            var index = new Random().Next(baseShapes.Length);

            this.ShapeArray = baseShapes[index];
            ShapeColor = colors[index];

            Random random = new Random();
            var d = random.Next(3);
            for (int r = 0; r < d; r++)
            {
                ChangeDirection();
            }

            Top = 0;
            Left = 4;
        }

        public void ChangeDirection()
        {
            if (Left + ShapeArray.GetLength(0) > 10)
            {
                return;
            }
            bool[,] temp = new bool[ShapeArray.GetLength(1), ShapeArray.GetLength(0)];


            for (var row = 0; row < ShapeArray.GetLength(0); row++)
            {
                for (var col = 0; col < this.ShapeArray.GetLength(1); col++)
                {
                    temp[col, ShapeArray.GetLength(0) - 1 - row] = this.ShapeArray[row, col];
                }
            }
            ShapeArray = temp;

        }

        public bool IsFull()
        {
            if (Top == 0)
            {
                for (var row = 0; row < ShapeArray.GetLength(0); row++)
                {
                    for (var col = 0; col < ShapeArray.GetLength(1); col++)
                    {
                        if (this.ShapeArray[row, col] && GridData[Top + row + 1, Left + col])
                            return true;
                    }
                }
            }
            return false;
        }

        public void MoveDown()
        {
            if (!this.IsBottom())
            {
                Top += 1;
            }
            else
            {
                for (var row = 0; row < ShapeArray.GetLength(0); row++)
                {
                    for (var col = 0; col < ShapeArray.GetLength(1); col++)
                    {
                        if (ShapeArray[row, col])
                            this.GridData[Top + row, Left + col] = true;
                    }
                }

                ReachedBottom?.Invoke();

                RandomShape();
            }
        }

        public bool IsBlock()
        {
            for (var row = 0; row < ShapeArray.GetLength(0); row++)
            {
                for (var col = 0; col < ShapeArray.GetLength(1); col++)
                {
                    //if (this.shapeGrid[row][col] == 1 && this.grid[this.top + row + 1][this.left + col] == 1)
                    //    return true;
                    if (Left > 0 && ShapeArray[row, col] && GridData[Top + row, Left + col - 1])
                        return true;
                    if ((Left + ShapeArray.GetLength(1)) < 10 && ShapeArray[row, col] && GridData[Top + row, Left + col + 1])
                        return true;
                }
            }
            return false;
        }

        public event Action ReachedBottom;
        public void MoveLeft()
        {
            if (this.Left > 0 && !this.IsBlock())
            {
                this.Left--;
            }
        }

        public void MoveRight()
        {
            if (Left + ShapeArray.GetLength(1) < 10 && !this.IsBlock())
            {
                this.Left++;
            }
        }

        public void Drop()
        {
            int slen = ShapeArray.GetLength(0);
            int ntop = 19 - ShapeArray.GetLength(0);
            for (var row = Top; row < ntop; row++)
            {
                for (var scol = 0; scol < ShapeArray.GetLength(1); scol++)
                {
                    if ((ShapeArray[slen - 1, scol]
                        && GridData[row + slen, Left + scol]))
                    {
                        Top = row;
                        MoveDown();
                        return;
                    }
                }
            }
            Top = ntop;
            MoveDown();
            return;
        }

        public void MoveQuickly()
        {
            if (!this.IsBottom())
            {
                this.Top += 1;
            }
        }
        bool IsBottom()
        {
            if (Top + ShapeArray.GetLength(0) < 20)
            {
                for (var row = 0; row < ShapeArray.GetLength(0); row++)
                {
                    for (var col = 0; col < ShapeArray.GetLength(1); col++)
                    {
                        if (this.ShapeArray[row, col] && GridData[Top + row + 1, Left + col])
                            return true;
                    }
                }
            }
            return Top + ShapeArray.GetLength(0) == 20;
        }
    }

}
